#include <Conversion.h>
#include <Object.h>
#include <Index.h>
#include <vector>

using namespace doste;

OID doste::IntToObj(int value)
{
	OID res;
	res.type = OID_TYPE_INTEGER;
	res.value = value;
	return res;
}

int doste::ObjToInt(OID o)
{
	if (o.type == OID_TYPE_INTEGER)
		return o.value;
	else if (o.type == OID_TYPE_FLOAT)
		return (int)*(float*)&o.value;
	else if (o.type == OID_TYPE_CHAR)
		return (int)o.value;
	else if (o == True)
		return 1;
	else
		return 0;
}

OID doste::CharToObj(char value)
{
	OID res;
	res.type = OID_TYPE_CHAR;
	res.value = value;
	return res;
}

char doste::ObjToChar(OID o)
{
	if (o.type == OID_TYPE_CHAR)
		return o.value;
	else if (o.type == OID_TYPE_INTEGER)
		return o.value;
	else
		return 0;
}

OID doste::FloatToObj(float value)
{
	OID res;
	res.type = OID_TYPE_FLOAT;
	res.value = *(int*)&value;
	return res;
}

float doste::ObjToFloat(OID o)
{
	if (o.type == OID_TYPE_FLOAT)
		return *(float*)&o.value;
	else if (o.type == OID_TYPE_INTEGER)
		return (float)o.value;
	else if (o.type == OID_TYPE_CHAR)
		return (float)o.value;
	else if (o == True)
		return 1.0;
	else
		return 0.0;
}

OID doste::BoolToObj(bool value)
{
	if (value)
		return True;
	else
		return False;
}

bool doste::ObjToBool(OID o)
{
	if (o == True)
		return true;
	else if (o.type == OID_TYPE_INTEGER)
		if (o.value > 0)
			return true;
		else
			return false;
	else if (o.type == OID_TYPE_FLOAT)
		if (*(float*)&o.value > 0.0000000001)
			return true;
		else
			return false;
	else
		return false;
}

char *doste::OIDToString(OID o, char *buffer, int size)
{
	switch (o.type)
	{
		case OID_TYPE_INTEGER:		sprintf(buffer, "%i", o.value); return buffer;
		case OID_TYPE_FLOAT:		sprintf(buffer, "%f", ObjToFloat(o)); return buffer;
		case OID_TYPE_INDEX:		strcpy(buffer, Index::Lookup(o)); return buffer;
		case OID_TYPE_SPECIAL:		strcpy(buffer, Index::Lookup(o)); return buffer;
		case OID_TYPE_CHAR:			buffer[0] = '\'';
									buffer[1] = (char)o.value;
									buffer[2] = '\'';
									buffer[3] = 0;
									return buffer;

		default:	sprintf(buffer, "[%i:%i]", (int)o.type, o.value); return buffer;
	}

	return 0;
}

OID doste::StrToObj(const char *str)
{
	if (str == 0)
		return Null;

	Object *res = new Object();
	res->Open();
	int pos = 0;

	while (str[pos] != 0)
	{
		if (str[pos] != '"' || str[pos+1] != 0)
		{
			res->Set(IntToObj(pos), CharToObj(str[pos]));
			pos++;
		} else
			break;
	}

	res->Set(Size, IntToObj(pos));

	res->Close(false);
	return res->GetOID();
}

char *doste::ObjToStr(OID o, char *buf, int bufsize, bool quotes)
{
	if (o.type != OID_TYPE_BASE)
		return OIDToString(o, buf, bufsize);

	Object *obj = Object::Lookup(o);
	if (obj == 0)
		return 0;
	obj->Open();

	OID aoid = Null;
	aoid = obj->Get(Size);

	if (aoid.type != OID_TYPE_INTEGER || obj->Get(IntToObj(0)).type != OID_TYPE_CHAR)
	{
		obj->Close();
		return OIDToString(o, buf, bufsize);
	}

	int size = ObjToInt(aoid);
	int pos = 0;

	if (quotes)
	{
		buf[pos] = '"';
		pos++;
	}

	for (int i=0; i<size; i++)
	{
		aoid = obj->Get(IntToObj(i));

		if (aoid.type == OID_TYPE_CHAR)
		{
			buf[pos] = ObjToChar(aoid);
			pos++;
		} else if (aoid.type == OID_TYPE_INDEX) {
			//aoid = obj->GetAttribute(attrib);
			//ObjToStr(attrib, &buf[pos], bufsize-pos);
			//while(buf[pos] != 0 && pos < bufsize)
			//	pos++;
		} else
		{
			OIDToString(aoid, buf, bufsize-pos);
			while(buf[pos] != 0 && pos < bufsize)
				pos++;
		}

		if (pos >= bufsize)
		{
			pos = bufsize-1;
			break;
		}
	}

	if (quotes)
	{
		buf[pos] = '"';
		pos++;
	}

	buf[pos] = 0;
	obj->Close();

	return buf;
}

OID doste::StringToOID(const char *str)
{
	OID result;

	switch (str[0])						//Check first character to get type
	{
		case '-':
		case '0':							//If any numerical digit
		case '1':
		case '2':
		case '3':
		case '4':
		case '5':
		case '6':
		case '7':
		case '8':
		case '9':	if (strchr(str, '.') == 0)			//Then check for a '.' for floating point
					{
						result.type = OID_TYPE_INTEGER;		//If not then its an integer
						result.value = atoi(str);			//So convert it to an integer
					} else {
						result = FloatToObj(atof(str));	//Otherwise it is so convert it to a float
					}
					break;

		case '\'':	result.type = OID_TYPE_CHAR;		//Its a character
					result.value = str[1];					//Use next character as character
					break;								//Should allow for escape characters!!!

		case '"':	//str[strlen(str)-1] = 0;					//Its a string
					result = StrToObj(&str[1]);	//So create a new string object for it
					break;

		//case LC:	result = TranslateOID(word); break;	//Its an explicit OID so convert

		default:	result = Index::Lookup(str);			//Otherwise it's an index label so look it up.
	}

	return result;
}

char *doste::DisplayObject(OID obj, char *buffer, int maxsize, int tabs)
{
	int pos = 0;
	vector<OID> *def;

	Object *object = Object::Lookup(obj);
	if (object != 0)
	{
		object->Open();
		buffer[pos] = '{';
		pos++;
		buffer[pos] = '\n';
		pos++;

		OID cur;
		int i = 0;
		cur = object->GetAttribute(i);
		bool first = true;

		while (cur != Error)
		{
			if (cur == Parent)
			{
				i++;
				cur = object->GetAttribute(i);
				continue;
			}

			if (!first)
			{
				buffer[pos] = ',';
				pos++;
				buffer[pos] = '\n';
				pos++;
			}

			first = false;

			for (int j=0; j<tabs; j++)
			{
				buffer[pos] = '\t';
				pos++;
			}
			buffer[pos] = '(';
			pos++;
			OIDToString(cur, &buffer[pos], maxsize-pos);
			pos += strlen(&buffer[pos]);
			buffer[pos] = ',';
			pos++;
			buffer[pos] = ' ';
			pos++;
			def = object->GetDefinition(cur);

			if (def != 0)
			{
				DefToString(def, &buffer[pos], maxsize-pos, tabs+1);
				pos += strlen(&buffer[pos]);

				buffer[pos] = ')';
				pos++;
				buffer[pos] = ',';
				pos++;
				buffer[pos] = '\n';
				pos++;

				for (int j=0; j<tabs; j++)
				{
					buffer[pos] = '\t';
					pos++;
				}

				buffer[pos] = '(';
				pos++;
				OIDToString(cur, &buffer[pos], maxsize-pos);
				pos += strlen(&buffer[pos]);
				buffer[pos] = ',';
				pos++;
				buffer[pos] = ' ';
				pos++;
			}

			if (object->Get(cur).type != OID_TYPE_BASE)
				ObjToStr(object->Get(cur), &buffer[pos], maxsize-pos, true);
			else
				DisplayObject(object->Get(cur), &buffer[pos], maxsize-pos, tabs+1);
			pos += strlen(&buffer[pos]);

			buffer[pos] = ')';
			pos++;

			i++;
			cur = object->GetAttribute(i);
		}

		buffer[pos] = '\n';
		pos++;
		for (int i=0; i<tabs; i++)
		{
			buffer[pos] = '\t';
			pos++;
		}
		buffer[pos] = '}';
		pos++;
		buffer[pos] = 0;
		pos++;

		object->Close();
	}

	return buffer;
}

char *doste::DefToString(vector<OID> *def, char *buffer, int maxsize, int tabs)
{
	int pos = 0;
	if (def == 0)
		return 0;
	
	if ((*def)[0].type == OID_TYPE_BASE)
		DisplayObject((*def)[0], buffer, maxsize, tabs);
	else
		OIDToString((*def)[0], buffer, maxsize);
	pos = strlen(buffer);
	for (vector<OID>::iterator i=def->begin()+1; i<def->end(); i++)
	{
		buffer[pos] = '(';
		pos++;
		if ((*i).type == OID_TYPE_DEF)
			DefToString((vector<OID>*)((*i).value), &buffer[pos], maxsize-pos);
		else
			ObjToStr(*i, &buffer[pos], maxsize-pos);
		pos += strlen(&buffer[pos]);
		buffer[pos] = ')';
		pos++;
	}
	buffer[pos] = 0;

	return buffer;
}

std::ostream &operator<<(std::ostream &in, const doste::OID &o)
{
	char buf[50];
	OIDToString(o,buf,50);
	in << buf;
	return in;
}
