#include <Index.h>
#include <Object.h>
#include <string.h>

int Index::lastindex = 0;
IndexEntry *Index::m_indexhash[INDEX_HASH_SIZE];

void Index::Init()
{
	for (int i=0; i<INDEX_HASH_SIZE; i++)							//Clear the hash table
		m_indexhash[i] = 0;
}

void Index::Final()
{
	//Delete all precompiled code and indexes.
	IndexEntry *cur, *temp;
	
	for (int i=0; i<INDEX_HASH_SIZE; i++)			//For every hash list
	{
		cur = m_indexhash[i];
		while (cur != 0)							//Go through all elements in the list
		{
			temp = cur->next;
			delete cur->name;						//Delete the name string
			delete cur;								//and delete the index entry
			cur = temp;
		}	
	}
}

char *Index::Lookup(OID o)
{
	IndexEntry *cur = 0;
	
	if (o == Null)
		return "null";

	if (o == Error)
		return "error";

	for (int i=0; i<INDEX_HASH_SIZE; i++)
	{
		cur = m_indexhash[i];
		while (cur != 0)
		{
			if (cur->value == o)
				return cur->name;
			cur = cur->next;
		}
	}

	return "UNKNOWN";
} 

OID Index::Add(const char *word)
{
	OID newoid;
	newoid.type = OID_TYPE_INDEX;
	newoid.value = lastindex;
	lastindex++;
	return Add(newoid, word);
}

OID Index::Add(OID o, const char *word)
{
	OID newoid = o;
	int hash = HashIndex(word);

	IndexEntry *cur = new IndexEntry;
	cur->next = m_indexhash[hash];
	m_indexhash[hash] = cur;
	cur->name = new char[strlen(word)+1];
	strcpy(cur->name, word);
	cur->value = newoid;
	return newoid;
}

OID Index::Lookup(const char *word)
{
	int hash = HashIndex(word);
	IndexEntry *cur = m_indexhash[hash];
	
	if (strcmp(word, "null") == 0)
		return Null;

	while (cur != 0)
	{
		if (strcmp(cur->name, word) == 0)
		{
			return cur->value;
		}
		cur = cur->next;
	}

	return Add(word);
}

inline int Index::HashIndex(const char *word)
{
	int res;
	res = word[0] * word[1];
	if (word[1] != 0)
		res += word[2];
	return res % INDEX_HASH_SIZE;
}
