#include <stdio.h>
#include <Object.h>
#include <Compiler.h>
#include <Index.h>
#include <Conversion.h>
#include <ObjectFactory.h>
#include <iostream>

using namespace doste;

Object *Object::system = 0;
bool Object::active = false;
OID Object::lastoid;
OID Object::lastvkey;
OID Object::currentobj = Null;
list<Object*> **Object::objects = 0;
ObjectNotifiable *Object::gnotif = 0;
list<Object*> *Object::recycled = 0;
unsigned int Object::m_numactiveobjects = 0;
pthread_t Object::qthread;
pthread_mutex_t Object::qmutex;
pthread_cond_t Object::qcond;
list<Object::SQueue*> Object::setqueue;
bool Object::iswaiting = true;
list<Object::SQueue*> Object::freequeue;

//--------- OBJECT -------------

Object::Object()
{
	oid = AllocateOID();
	exrefs = 0;
	//notifiables = new list<ObjectNotifiable*>();
	notifiables = 0;
	attribs = new list<RAttribute*>*[PAIR_HASH_SIZE];
	vattribs = 0;
	parent = Null;

	for (int i=0; i<PAIR_HASH_SIZE; i++)
		attribs[i] = 0;

	AddObject(this);
}

Object::Object(OID o)
{
	oid = o;
	exrefs = 0;
	//notifiables = new list<ObjectNotifiable*>();
	notifiables = 0;
	attribs = new list<RAttribute*>*[PAIR_HASH_SIZE];
	vattribs = 0;
	parent = Null;

	for (int i=0; i<PAIR_HASH_SIZE; i++)
		attribs[i] = 0;

	AddObject(this);
}

Object::~Object()
{
	if (notifiables != 0)
		delete notifiables;
	notifiables = 0;

	if (vattribs != 0)
	{
		list<VAttribute*> *temp = vattribs;
		vattribs = 0;
		for (list<VAttribute*>::iterator i=temp->begin(); i!=temp->end(); i++)
		{
			delete (*i)->vkey;
		}
		delete temp;
	}

	if (attribs != 0)
	{
		list<RAttribute*> *temp;
		
		for (int i=0; i<PAIR_HASH_SIZE; i++)
		{
			if (attribs[i] != 0)
			{
				temp = attribs[i];
				attribs[i] = 0;
				for (list<RAttribute*>::iterator j=temp->begin(); j != temp->end(); j++)
				{
					delete (*j)->value;
					delete *j;
				}
				delete temp;
			}
		}
	
		delete [] attribs;
		attribs = 0;
	}

	RemoveObject(this);
}

OID Object::Get(OID key)
{
	if (key == Parent)
	{
		return parent;
	}

	OID res;
	Observable *observ = LookupObservable(key);

	if (observ == 0)
	{
		if (key == Index::Lookup("compare"))
			return ObjectFactory::CreateCompare(oid)->GetOID();
		if (key == Index::Lookup("union"))
			return ObjectFactory::CreateUnion(oid)->GetOID();
		if (key == Index::Lookup("uniond"))
			return ObjectFactory::CreateUnionD(oid)->GetOID();

		key = Star;
		observ = LookupObservable(key);
	}

	if (observ == 0)
	{
		return Null;
	}

	res = observ->Get();
	return res;
}

unsigned char Object::GetFlags(OID key)
{
	Observable *observ = LookupObservable(key);
	if (observ == 0)
		return 0;
	return observ->GetFlags();
}

void Object::SetFlags(OID key, unsigned char flags)
{
	Observable *observ = LookupObservable(key);
	if (observ == 0)
		return;
	return observ->SetFlags(flags);
}

void Object::Set(OID key, OID value)
{
	if (!active)
		return;

	//pthread_mutex_lock(&qmutex);
	SQueue *pair = AllocateQueueEntry();
	//pthread_mutex_unlock(&qmutex);

	if (pair == 0)
	{
		active = false;
		/*pthread_mutex_lock(&qmutex);
		if (iswaiting)
		{
			iswaiting = false;
			pthread_cond_signal(&qcond);
		}
		pthread_mutex_unlock(&qmutex);*/

		return;
	}
	pair->key = key;
	pair->value = value;
	this->Open();
	pair->obj = this;

	/*pthread_mutex_lock(&qmutex);
	if (iswaiting)
	{*/
		setqueue.push_front(pair);
	/*	iswaiting = false;
		pthread_cond_signal(&qcond);
	} else {
		setqueue.push_front(pair);
	}
	pthread_mutex_unlock(&qmutex);*/

	if (iswaiting)
	{
		iswaiting = false;
		QueueProcessor(NULL);
		iswaiting = true;
	}
}

/*void Object::Set(OID key, vector<OID> *def)
{
	SQueue *pair = new SQueue;
	pair->key = key;
	pair->value.type = OID_TYPE_DEF;
	pair->value.value = (int)def;
	pair->obj = this;

	pthread_mutex_lock(&qmutex);
	if (setqueue.size() == 0 && iswaiting)
	{
		setqueue.push_front(pair);
		pthread_cond_signal(&qcond);
	} else {
		setqueue.push_front(pair);
	}
	pthread_mutex_unlock(&qmutex);
}*/

void *Object::QueueProcessor(void *p)
{
	//std::cout << "QPROC\n";
	SQueue *pair;
	Object *obj;
	OID key, value;
	//iswaiting = false;

	while(setqueue.size() != 0 && active)
	{
		/*pthread_mutex_lock(&qmutex);
		if (setqueue.size() == 0)
		{
			iswaiting = true;
			pthread_cond_wait(&qcond, &qmutex);
			iswaiting = false;
		}*/
		pair = setqueue.back();
		setqueue.pop_back();
		//pthread_mutex_unlock(&qmutex);

		obj = pair->obj;
		key = pair->key;
		value = pair->value;
		//pthread_mutex_lock(&qmutex);
		FreeQueueEntry(pair);
		//pthread_mutex_unlock(&qmutex);
		obj->QSet(key, value);
		obj->Close();
	}

	//iswaiting = false;
	//pthread_exit(NULL);
	return 0;
}

void Object::QSet(OID key, OID value)
{
	std::cout << "QSET: " << key << "=" << value << std::endl;

	if (key == Parent)
	{
		parent = value;
		return;
	}

	Observable *observ = LookupObservable(key);

	if (observ == 0)
	{
		RAttribute *attrib = new RAttribute;
		attrib->key = key;
		attrib->value = new Observable();
		observ = attrib->value;
		AddRAttribute(attrib);
	}

	if (observ->Set(value))
	{
		observ->NotifyDependants();
		if (notifiables != 0)
			for (list<ObjectNotifiable*>::iterator i=notifiables->begin(); i!=notifiables->end(); i++)
				(*i)->OnChange(oid, key, observ->Get());
		//Notify(key, forced);
	}

	//if (value == Null && observ->GetDefinition() != 0)
	//	RemoveRAttribute(key);
}

void Object::Set(vector<OID> *key, OID value)
{
	VAttribute *vattrib = new VAttribute;
	vattrib->vkey = new Observable();
	vattrib->vkey->Define(key);
	vattrib->key = GenVKey();
	AddVAttribute(vattrib);
	Set(vattrib->key, value);
}

void Object::Set(vector<OID> *key, vector<OID> *def)
{
	VAttribute *vattrib = new VAttribute;
	vattrib->vkey = new Observable();
	vattrib->vkey->Define(key);
	vattrib->key = GenVKey();
	AddVAttribute(vattrib);
	Set(vattrib->key, def);
}

void Object::Set(OID key, vector<OID> *def)
{
	Observable *observ = LookupObservable(key);

	if (observ == 0)
	{
		RAttribute *attrib = new RAttribute;
		attrib->key = key;
		attrib->value = new Observable();
		observ = attrib->value;
		AddRAttribute(attrib);
	}

	observ->Define(def);
	Notify(key, false);
}

vector<OID> *Object::GetDefinition(OID key)
{
	Observable *observ = LookupObservable(key);

	if (observ == 0)
	{
		return 0;
	}

	return observ->GetDefinition();
}

list<SDependant> *Object::GetDependants(OID key)
{
	Observable *observ = LookupObservable(key);

	if (observ == 0)
	{
		return 0;
	}

	return observ->GetDependants();
}

int Object::GetNumReferences()
{
	return exrefs;
}

void Object::Notify(OID key, bool forced)
{
	if (key == Parent)
		return;

	Observable *observ = LookupObservable(key);

	if (observ == 0)
	{
		key = Star;
		observ = LookupObservable(key);
		if (observ == 0)
			return;
	}

	/*if (forced)
	{
		observ->NotifyDependants();
		if (notifiables != 0)
			for (list<ObjectNotifiable*>::iterator i=notifiables->begin(); i!=notifiables->end(); i++)
				(*i)->OnChange(oid, key, observ->Get());
		return;
	}*/

	observ->OnNotify(oid, key, this);
	//if (notifiables != 0)
	//	for (list<ObjectNotifiable*>::iterator i=notifiables->begin(); i!=notifiables->end(); i++)
	//		(*i)->OnChange(oid, key, observ->Get());
}

void Object::AddDependant(OID key, OID obj, OID attr)
{
	if (key == Parent)
		return;

	OID oldkey;
	Observable *observ = LookupObservable(key);

	if (observ == 0)
	{
		if (key == Index::Lookup("compare"))
			return;
		if (key == Index::Lookup("union"))
			return;
		if (key == Index::Lookup("uniond"))
			return;

		oldkey = key;
		key = Star;
		observ = LookupObservable(key);
		if (observ == 0)
		{
			RAttribute *attrib = new RAttribute;	//This constructs unwanted attributes.
			attrib->key = oldkey;
			attrib->value = new Observable();
			observ = attrib->value;
			AddRAttribute(attrib);
		}
	}

	observ->AddDependant(obj, attr);

	if (notifiables != 0)
		for (list<ObjectNotifiable*>::iterator i=notifiables->begin(); i!=notifiables->end(); i++)
			(*i)->OnAddDependant(key, obj, attr);
}

void Object::RemoveDependant(OID key, OID obj, OID attr)
{
	if (key == Parent)
		return;

	Observable *observ = LookupObservable(key);

	if (observ == 0)
	{
		key = Star;
		observ = LookupObservable(key);
		if (observ == 0)
			return;
	}

	observ->RemoveDependant(obj, attr);
}

void Object::Open()
{
	exrefs++;
	char buf[50];
	OIDToString(oid, buf, 50);
}

void Object::Close(bool garbageable)
{
	exrefs--;
	char buf[50];
	OIDToString(oid, buf, 50);
	if (exrefs == 0 && oid.type != OID_TYPE_SPECIAL)
	{
		if (garbageable)
		{
			//printf("GARBAGE\n");
			if (notifiables != 0)
				for (list<ObjectNotifiable*>::iterator i=notifiables->begin(); i!=notifiables->end(); i++)
					(*i)->OnGarbage(oid);
			delete this;
			//RecycleObject(this);
		}
	}
}

Observable *Object::LookupObservable(OID name)
{
	int hash = HashOID(name, PAIR_HASH_SIZE);
	list<RAttribute*> *cur = attribs[hash];

	if (cur != 0)
	{
		for (list<RAttribute*>::iterator i=cur->begin(); i != cur->end(); i++)
		{
			if ((*i)->key == name)
			{
				return (*i)->value;
			}
		}
	}

	if (vattribs != 0)
	{
		for (list<VAttribute*>::iterator i=vattribs->begin(); i!=vattribs->end(); i++)
		{
			if ((*i)->vkey->Get(oid, true) == name)
			{
				return LookupObservable((*i)->key);
			}
		}
	}

	return 0;
}

void Object::AddRAttribute(RAttribute *attrib)
{
	int hash = HashOID(attrib->key, PAIR_HASH_SIZE);
	if (attribs[hash] == 0)
		attribs[hash] = new list<RAttribute*>;
	list<RAttribute*> *nlist = attribs[hash];

	nlist->push_front(attrib);
}

void Object::RemoveRAttribute(OID key)
{
	int hash = HashOID(key, PAIR_HASH_SIZE);
	if (attribs[hash] == 0)
		return;
	for (list<RAttribute*>::iterator i=attribs[hash]->begin(); i!=attribs[hash]->end(); i++)
	{
		if ((*i)->key == key)
		{
			delete (*i)->value;
			delete (*i);
			attribs[hash]->erase(i);
			return;
		}
	}
}

void Object::AddVAttribute(VAttribute *attrib)
{
	if (vattribs == 0)
		vattribs = new list<VAttribute*>();

	vattribs->push_front(attrib);
}

OID Object::GetAttribute(int index)
{
	for (int i=0; i<PAIR_HASH_SIZE; i++)
	{
		if (attribs[i] == 0)
			continue;

		for (list<RAttribute*>::iterator j=attribs[i]->begin(); j != attribs[i]->end(); j++)
		{
			if (index == 0 && (*j)->key.type != OID_TYPE_VKEY)
				return (*j)->key;

			if ((*j)->key.type != OID_TYPE_VKEY)
				index--;
		}
	}

	if (vattribs)
	{
		for (list<VAttribute*>::iterator i=vattribs->begin(); i!=vattribs->end(); i++)
		{
			if (index == 0)
				return (*i)->vkey->Get(oid, true);
			index--;
		}
	}

	return Error;
}

void Object::AddNotifiable(ObjectNotifiable *notif, unsigned int flags)
{
	if (notifiables == 0)
		notifiables = new list<ObjectNotifiable*>();
	notifiables->push_front(notif);
}

void Object::RemoveNotifiable(ObjectNotifiable *notif)
{
	if (notifiables == 0)
		return;
	notifiables->remove(notif);
}

int Object::HashOID(OID o, int size)
{
	return o.value % size;
}

//------ STATIC MANAGEMENT FUNCTIONS -------

OID Object::AllocateOID()
{
	lastoid.value++;
	return lastoid;
}

OID Object::GenVKey()
{
	lastvkey.value++;
	return lastvkey;
}

void Object::AddObject(Object *obj)
{
	int hash = HashOID(obj->GetOID(), GLOBAL_HASH_SIZE);
	
	if (objects[hash] == 0)
		objects[hash] = new list<Object*>;

	objects[hash]->push_front(obj);

	m_numactiveobjects++;
}

void Object::RemoveObject(Object *obj)
{
	int hash = HashOID(obj->GetOID(), GLOBAL_HASH_SIZE);
	
	list<Object*> *cur = objects[hash];

	if (cur == 0)
		return;
	
	for (list<Object*>::iterator i=cur->begin(); i!=cur->end(); i++)
	{
		if ((*i) == obj)
		{
			//delete *i;
			cur->erase(i);
			m_numactiveobjects--;
			return;
		}
	}
}

/*void Object::RecycleObject(Object *obj)
{
	if (recycled->size() < 100)
	{
		RemoveObject(obj);
		recycled->push_front(obj);
	} else
		delete obj;
}

Object *Object::NewObject()
{
	Object *res;

	if (!recycled->empty())
	{
		res = recycled->front();
		recycled->pop_front();
		res->Reset(AllocateOID());
		return res;
	} else
	{
		return new Object();
	}
}

Object *Object::NewObject(OID o)
{
	Object *res;

	if (!recycled->empty())
	{
		res = recycled->front();
		recycled->pop_front();
		res->Reset(o);
		return res;
	} else
	{
		return new Object(o);
	}
}*/

Object::SQueue *Object::AllocateQueueEntry()
{
	//std::cout << "AllocateQueueEntry: " << freequeue.size() << std::endl;
	if (freequeue.size() == 0)
		return 0;
	SQueue *q = freequeue.back();
	freequeue.pop_back();
	return q;
}

void Object::FreeQueueEntry(Object::SQueue *q)
{
	//std::cout << "FreeQueueEntry: " << freequeue.size() << "\n";
	freequeue.push_front(q);
}

Object *Object::Lookup(OID o)
{
	if (o == Current)
		return Lookup(currentobj);

	int hash = HashOID(o, GLOBAL_HASH_SIZE);
	list<Object*> *cur = objects[hash];

	if (cur != 0)
	{
		for (list<Object*>::iterator i=cur->begin(); i != cur->end(); i++)
		{
			if ((*i)->GetOID() == o)
			{
				return *i;
			}
		}
	}

	if (o.type == OID_TYPE_INTEGER || o.type == OID_TYPE_FLOAT)
	{
		return ObjectFactory::CreateNumber(o);
	}

	return 0;
}

Object *Object::GetSystem()
{
	return system;
}

unsigned int Object::GetNumObjects()
{
	return m_numactiveobjects;
}

void Object::SetCurrent(OID current)
{
	currentobj = current;
}

OID Object::GetCurrent()
{
	return currentobj;
}

void Object::Init()
{
	lastoid.type = OID_TYPE_BASE;
	lastoid.value = 0;
	lastvkey.type = OID_TYPE_VKEY;
	lastvkey.value = 0;
	objects = new list<Object*>*[GLOBAL_HASH_SIZE];

	for (int i=0; i<GLOBAL_HASH_SIZE; i++)
		objects[i] = 0;

	for (int i=0; i<MAX_QUEUE_SIZE; i++)
	{
		freequeue.push_front(new SQueue);
	}

	active = true;

	//pthread_mutex_init(&qmutex, NULL);
	//pthread_cond_init(&qcond, NULL);
	//pthread_create(&qthread, NULL, QueueProcessor, NULL);

	system = new Object(System);
	new Object(True);
	new Object(False);

	ObjectFactory::Init();
}

void Object::Final()
{
	//pthread_cancel(qthread);
	//pthread_join(qthread, NULL);

	ObjectFactory::Final();
	//Destroy all objects.
	list<Object*> *temp;

	for (int i=0; i<GLOBAL_HASH_SIZE; i++)
	{
		if (objects[i] != 0)
		{
			temp = objects[i];
			objects[i] = 0;
			for (list<Object*>::iterator j=temp->begin(); j != temp->end(); j++)
			{
				delete *j;
			}
			delete temp;
		}
	}

	active = false;

	SQueue *q;
	for (list<SQueue*>::iterator i=freequeue.begin(); i!=freequeue.end(); i++)
	{
		delete *i;
		//freequeue.erase(i);
	}
	for (list<SQueue*>::iterator i=setqueue.begin(); i!=setqueue.end(); i++)
	{
		delete *i;
		//setqueue.erase(i);
	}

	delete [] objects;
}

