#include <Observable.h>
#include <Object.h>
#include <Compiler.h>
#include <Conversion.h>
#include <iostream>

using namespace doste;

bool operator==(SDependant d1, SDependant d2)
{
	if (d1.object == d2.object && d1.attrib == d2.attrib) return true;
	return false;
}

Observable::Observable()
{
	definition = 0;
	dependants = 0;
	selfdep = false;
	value = Null;
	flags = 0;
}

Observable::~Observable()
{
	if (definition != 0)
	{
		Object *val = Object::Lookup((*definition)[0]);
		if (val != 0)
		{
			val->Close();
		}
	}

	Object *val = Object::Lookup(value);
	if (val != 0)
	{
		val->Close();
	}

	if (definition != 0)
		delete definition;

	if (dependants != 0)
		delete dependants;
}

unsigned char Observable::GetFlags()
{
	return flags;
}

void Observable::SetFlags(unsigned char flags)
{
	this->flags = flags;
}

bool Observable::Set(OID value)
{
	Object *val;

	if (value == this->value)
		return false;

	/* Garbage Collection */
	if (value != this->value)
	{
		val = Object::Lookup(this->value);
		if (val != 0)
		{
			val->Close();
		}
	}

	OID oldval = this->value;
	this->value = value;

	/* Garbage Collection */
	if (value != oldval)
	{
		val = Object::Lookup(this->value);
		if (val != 0)
		{
			val->Open();
		}

		/*val = Object::Lookup(obj);
		if (val != 0)
		{
			val->Open();
			forced = true;
			val->Notify(key);
			val->Close();
		}*/

		//NotifyDependants();

		return true;
	}

	return false;
}

OID Observable::Get(OID obj, bool eval)
{
	if (!eval)
		return value;
	else
	{
		value = Evaluate(obj, Null);
		return value;
	}
}

void Observable::Define(vector<OID> *def)
{
	Object *val;

	if (def == 0)
		return;

	if (definition != 0)
	{
		/* Garbage collection on first object in definition */
		val = Object::Lookup((*definition)[0]);
		if (val != 0)
		{
			val->Close();
		}
		delete definition;
	}

	definition = def;

	/* Garbage Collection */
	val = Object::Lookup((*def)[0]);
	if (val != 0)
	{
		val->Open();
	}

	/*val = Object::Lookup(obj);
	if (val != 0)
	{
		val->Open();
		val->Notify(key);
		val->Close();
	}*/
}

vector<OID> *Observable::GetDefinition()
{
	return definition;
}

list<SDependant> *Observable::GetDependants()
{
	return dependants;
}

void Observable::AddDependant(OID obj, OID attrib)
{
	if (dependants == 0)
		dependants = new list<SDependant>;

	SDependant dep;
	dep.object = obj;
	dep.attrib = attrib;
	dependants->remove(dep);
	dependants->push_back(dep);
}

void Observable::RemoveDependant(OID obj, OID attrib)
{
	SDependant dep;
	dep.object = obj;
	dep.attrib = attrib;

	if (dependants != 0)
	{
		dependants->remove(dep);
		if (dependants->empty())
		{
			delete dependants;
			dependants = 0;
		}
	}
}

bool Observable::OnNotify(OID obj, OID key, void *thi)
{
	OID oldvalue;
	OID newvalue;
	Object *val;
	Object *object = (Object*)thi;

	/*if (forced)
	{
		forced = false;
		NotifyDependants();
		return true;
	}*/

	if (definition != 0)
	{
	//while (true)
	//{
		//selfdep = false;
		oldvalue = value;
		newvalue = Evaluate(obj, key, definition);
		if (newvalue != oldvalue)
		{
			//Set(newvalue);
			//NotifyDependants();
			object->Set(key, newvalue);
		}

		/*if (value != oldvalue)
		{
			val = Object::Lookup(oldvalue);
			if (val != 0)
			{
				val->Close();
			}

			val = Object::Lookup(value);
			if (val != 0)
			{
				val->Open();
			}
		} else
			return false;*/

		if (newvalue == oldvalue)
			return false;

		//if (!selfdep)
		//	return true;
	//}
	} else
	{
		//NotifyDependants();
	}

	return true;
}

void Observable::NotifyDependants()
{
	SDependant dep;
	Object *object;

	if (dependants == 0)
	{
		return;
	}

	int size = dependants->size();

	for (int i=0; i<size; i++)
	{
		dep = dependants->front();
		dependants->pop_front();
		object = Object::Lookup(dep.object);
		if (object != 0)
			object->Notify(dep.attrib, false);
	}
}

OID Observable::Evaluate(OID obj, OID key)
{
	return Evaluate(obj, key, definition);
}

OID Observable::Evaluate(OID object, OID key, vector<OID> *def)
{
	OID res = (*def)[0];
	Object *obj;
	Object *lastobj = 0;
	Observable *attrib;
	OID temp;

	if (res == This)
		res = object;

	for (int i=1; i<def->size(); i++)
	{
		obj = Object::Lookup(res);
		if (obj != 0)
		{
			obj->Open();

			if (lastobj != 0)
				lastobj->Close();		//Delayed close so garbage does not invalidate result yet.

			temp = (*def)[i];
			if (temp.type == OID_TYPE_DEF)
			{
				temp = Evaluate(object, key, (vector<OID>*)(temp.value));
			}
			//if (res != object || temp != key)
				obj->AddDependant(temp, object, key);
			//else
			//	selfdep = true;
			res = obj->Get(temp);

			lastobj = obj;
			//obj->Close();
		} else
		{
			res = Null;
			break;
		}
	}

	if (lastobj != 0)
		lastobj->Close();

	return res;
}
