#include <qpushbutton.h>
#include <qgroupbox.h>
#include <qtextedit.h>
#include <qlayout.h>
#include <AttributeWindow.h>
#include <Index.h>
#include <Conversion.h>
#include <Syntax.h>
#include <iostream>

using namespace doste;

AttributeWindow::AttributeWindow(OID obj, OID par, int num, QWidget *parent)
	: QWidget(parent)
{
	this->obj = obj;
	this->parent = par;
	this->num = num;

	definition = new QTextEdit(this);
	definition->setFont(QFont(QString("Courier New"), 10));
	definition->setAutoFormatting(QTextEdit::AutoNone);
	definition->setTabStopWidth(30);
	definition->setWordWrap(QTextEdit::NoWrap);
	Syntax *syn = new Syntax(definition);
	aobj = Null;
	aattrib = Null;

	Object *object = Object::Lookup(obj);
	if (object != 0)
	{
		object->Open();
		object->AddNotifiable(this, 0);

		int i=0;
		OID cur = object->GetAttribute(i);
		while (cur != Error)
		{
			OnChange(obj, cur, object->Get(cur));
			i++;
			cur = object->GetAttribute(i);
		}

		object->Close();
	}

	QVBoxLayout *mainlayout = new QVBoxLayout(this, 5);
	QGroupBox *details = new QGroupBox("Attribute Details", this);
	mainlayout->addWidget(details);
	mainlayout->addWidget(definition);
	QGridLayout *gridlayout = new QGridLayout(details, 2, 2, 20);
	gridlayout->addWidget(new QLabel("Name: ", details), 0, 0);
	attribname = new QLabel("null", details);
	gridlayout->addWidget(attribname, 0, 1);
	gridlayout->addWidget(new QLabel("Value: ", details), 1, 0);
	attribvalue = new QLineEdit("null", details);
	attribvalue->resize(150, attribvalue->height());
	connect(attribvalue, SIGNAL(returnPressed()), this, SLOT(updateValue()));
	gridlayout->addWidget(attribvalue, 1, 1);

	QHBoxLayout *buttonlayout = new QHBoxLayout(mainlayout);
	buttonlayout->setAlignment(AlignLeft);
	
	QPushButton *accept = new QPushButton(QIconSet(QPixmap("icons/accept.png")), "Accept", this);
	buttonlayout->addWidget(accept);
	connect(accept, SIGNAL(clicked()), this, SLOT(accept()));
	QPushButton *trace = new QPushButton(QIconSet(QPixmap("icons/trace.png")), "Trace", this);
	buttonlayout->addWidget(trace);
	QPushButton *monitor = new QPushButton(QIconSet(QPixmap("icons/monitor.png")), "Monitor", this);
	buttonlayout->addWidget(monitor);
	QPushButton *close = new QPushButton(QIconSet(QPixmap("icons/cancel.png")), "Close", this);
	buttonlayout->addWidget(close);
	connect(close, SIGNAL(clicked()), this, SLOT(close()));
}

AttributeWindow::~AttributeWindow()
{

}

void AttributeWindow::Update()
{
	vector<OID> *def;
	char *buf = new char[10000];

	char buf2[50];
	OIDToString(aattrib, buf2, 50);
	attribname->setText(buf2);

	Object *object = Object::Lookup(aobj);
	if (object != 0)
	{
		object->Open();
		OIDToString(object->Get(aattrib), buf2, 50);
		attribvalue->setText(buf2);
		def = object->GetDefinition(aattrib);
		if (def != 0)
		{
			DefToString(def, buf, 10000, 0);
			definition->setText(buf);
		}
		object->Close();
	}

	delete [] buf;
}

void AttributeWindow::OnChange(OID obj, OID key, OID value)
{
	if (key == Index::Lookup("object"))
	{
		aobj = value;
		if (aattrib != Null)
		{
			Update();
		}
	}
	else if (key == Index::Lookup("attribute"))
	{
		aattrib = value;

		if (aobj != Null)
		{
			Update();
		}
	}
}

void AttributeWindow::OnGarbage(OID obj)
{
	delete this;
}

void AttributeWindow::accept()
{
	if (aobj != Null && aattrib != Null)
	{
		char buf[50];
		char buf2[50];
		OIDToString(aobj, buf, 50);
		OIDToString(aattrib, buf2, 50);
		QString dtext = QString().sprintf("%s(%s) = %s;", buf, buf2, definition->text().ascii());
		Compiler::Execute(dtext);
	}
}

void AttributeWindow::close()
{
	Object *object = Object::Lookup(parent);
	if (object != 0)
	{
		object->Open();
		object->Set(IntToObj(num), Null);
		object->Close();
	}
}

void AttributeWindow::updateValue()
{
	Object *object = Object::Lookup(aobj);
	if (object != 0)
	{
		object->Open();
		object->Set(aattrib, StringToOID(attribvalue->text()));
		object->Close();
	}
}
 
