#include <GUIFactory.h>
#include <objects/DButton.h>
#include <objects/DGroupBox.h>
#include <objects/DLineEdit.h>
#include <objects/DLabel.h>
#include <objects/DNone.h>
#include <objects/DWindow.h>
#include <objects/DOpenGL.h>
#include <Object.h>
#include <Conversion.h>
#include <Index.h>

int GUIFactory::IdentifyType(OID obj, OID key)
{
	if (obj.type == OID_TYPE_INTEGER)
		return OBJTYPE_INTEGER;
	if (obj.type == OID_TYPE_FLOAT)
		return OBJTYPE_FLOAT;
	if (obj.type == OID_TYPE_CHAR)
		return OBJTYPE_CHAR;
	if (obj == Null)
		return OBJTYPE_NULL;
	if (obj == Current)
		return OBJTYPE_CURRENT;
	if (obj == True || obj == False)
		return OBJTYPE_BOOLEAN;
	if (obj == System)
		return OBJTYPE_SYSTEM;
	if (obj == Device_Screen)
		return OBJTYPE_DEV_SCREEN;
	if (obj == Device_Console)
		return OBJTYPE_DEV_CONSOLE;
	if (obj == Device_Core)
		return OBJTYPE_DEV_CORE;
	if (obj == Device_Clock)
		return OBJTYPE_DEV_CLOCK;
	if (obj == Device_Ide)
		return OBJTYPE_DEV_IDE;
	if (obj.type == OID_TYPE_INDEX)
		return OBJTYPE_INDEX;
	if (obj.type == OID_TYPE_VKEY)
		return OBJTYPE_VKEY;
	if (obj.type == OID_TYPE_SPECIAL)
		return OBJTYPE_SPECIAL;

	Object *object = Object::Lookup(obj);
	if (object != 0)
	{
		object->Open();
		
		if (object->Get(Index::Lookup("size")).type == OID_TYPE_INTEGER)
			if (object->Get(IntToObj(0)).type == OID_TYPE_CHAR)
				return OBJTYPE_STRING;

		if (object->Get(Index::Lookup("r")).type == OID_TYPE_INTEGER)
			if (object->Get(Index::Lookup("g")).type == OID_TYPE_INTEGER)
				if (object->Get(Index::Lookup("b")).type == OID_TYPE_INTEGER)
					return OBJTYPE_COLOR;

		if (key == Index::Lookup("font"))
		{
			if (object->Get(Index::Lookup("name")).type == OID_TYPE_BASE)
				return OBJTYPE_FONT;
			if (object->Get(Index::Lookup("size")).type == OID_TYPE_INTEGER)
				return OBJTYPE_FONT;
			if (object->Get(Index::Lookup("bold")) != Null)
				return OBJTYPE_FONT;
		}

		OID temp = object->Get(Index::Lookup("widget"));
		if (temp == Index::Lookup("window"))
			return OBJTYPE_WINDOW;
		if (temp == Index::Lookup("button"))
			return OBJTYPE_BUTTON;
		if (temp == Index::Lookup("label"))
			return OBJTYPE_LABEL;
		if (temp == Index::Lookup("groupbox"))
			return OBJTYPE_GROUPBOX;
		if (temp == Index::Lookup("lineedit"))
			return OBJTYPE_LINEEDIT;
		if (temp == Index::Lookup("opengl"))
			return OBJTYPE_OPENGL;

		temp = object->Get(Index::Lookup("entity"));
		if (temp == Index::Lookup("polygon"))
			return OBJTYPE_POLYGON;

		object->Close();
	}

	if (obj.type == OID_TYPE_BASE)
		return OBJTYPE_OBJECT;

	return OBJTYPE_UNKNOWN;
}

DWidget *GUIFactory::CreateWidget(OID type, OID par, OID key, OID obj, QWidget *parent)
{
	DWidget *widget = 0;

	if (type == Index::Lookup("window"))
		widget = new DWindow(par,key,obj, parent);
	else if (type == Index::Lookup("button"))
		widget = new DButton(par,key,obj, parent);
	else if (type == Index::Lookup("groupbox"))
		widget = new DGroupBox(par,key,obj, parent);
	else if (type == Index::Lookup("lineedit"))
		widget = new DLineEdit(par,key,obj, parent);
	else if (type == Index::Lookup("label"))
		widget = new DLabel(par,key,obj, parent);
	else if (type == Index::Lookup("none"))
		widget = new DNone(par,key,obj, parent);
	else if (type == Index::Lookup("opengl"))
		widget = new DOpenGL(par,key,obj, parent);

	return widget;
}

void GUIFactory::CreateFont(OID obj, QFont &font)
{
	Object *object = Object::Lookup(obj);
	if (object != 0)
	{
		object->Open();
		OID temp = object->Get(Index::Lookup("name"));
		if (temp != Null)
		{
			char buf[50];
			ObjToStr(temp, buf, 49);
			font.setFamily(buf);
		}

		temp = object->Get(Index::Lookup("size"));
		if (temp != Null)
			font.setPointSize(ObjToInt(temp));
		temp = object->Get(Index::Lookup("bold"));
		if (temp == True)
			font.setBold(true);
		else
			font.setBold(false);
		object->Close();
	}
}

void GUIFactory::CreateColor(OID obj, QColor &color)
{
	Object *object = Object::Lookup(obj);
	if (object != 0)
	{
		object->Open();
		if (object->Get(Index::Lookup("size")).type == OID_TYPE_INTEGER)
		{
			if (object->Get(IntToObj(0)).type == OID_TYPE_CHAR)
			{
				char buf[50];
				ObjToStr(obj, buf, 49);
				color.setNamedColor(buf);
				object->Close();
				return;
			}
		}

		OID r = object->Get(Index::Lookup("r"));
		OID g = object->Get(Index::Lookup("g"));
		OID b = object->Get(Index::Lookup("b"));

		color.setRgb(ObjToInt(r), ObjToInt(g), ObjToInt(b));

		object->Close();
	}
}
