#include <InputWindow.h>
#include <iostream>
#include <qstring.h>

InputWindow::InputWindow(QWidget *parent)
	: QWidget(parent)
{
	input = new QTextEdit(this);
	input->setFont(QFont(QString("Courier New"), 12));
	input->setAutoFormatting(QTextEdit::AutoNone);
	input->setTabStopWidth(30);
	input->setWordWrap(QTextEdit::NoWrap);
	syn = new Syntax(input);

	mainlayout = new QVBoxLayout(this, 2);
	mainlayout->addWidget(input);

	buttonlayout = new QHBoxLayout(mainlayout);
	buttonlayout->setAlignment(AlignLeft);
	accept = new QPushButton(QIconSet(QPixmap("icons/accept.png")), "Accept", this);
	accept->setMaximumWidth(120);
	QWidget::connect(accept, SIGNAL(clicked()), this, SLOT(acceptclick()));
	buttonlayout->addWidget(accept);

	clear = new QPushButton("Clear", this);
	clear->setMaximumWidth(120);
	QWidget::connect(clear, SIGNAL(clicked()), input, SLOT(clear()));
	buttonlayout->addWidget(clear);
}

InputWindow::~InputWindow()
{
	delete clear;
	delete accept;
	delete input;
	delete syn;
	delete buttonlayout;
	delete mainlayout;
}

void InputWindow::acceptclick()
{
	Compiler::Execute((const char*)input->text());
}

void InputWindow::LoadFile(QString &file)
{
	FILE *fh = fopen(file, "r");
	if (fh == 0)
		return;
	QTextStream *ts = new QTextStream(fh, IO_ReadOnly);
	input->setText(ts->read());
	delete ts;
	fclose(fh);
}

void InputWindow::SaveFile(QString &file)
{
	FILE *fh = fopen(file, "w");
	if (fh == 0)
		return;
	QTextStream *ts = new QTextStream(fh, IO_WriteOnly);
	*ts << input->text();
	delete ts;
	fclose(fh);
}
