#include <MainForm.h>
#include <qapplication.h>
#include <iostream>
#include <Compiler.h>
#include <Index.h>
#include <GUIFactory.h>
#include <Conversion.h>
#include <qtoolbutton.h>
#include <AttributeWindow.h>

using namespace doste;

extern QApplication *app;

MainForm::MainForm(QWidget *parent, const char *name)
	: QMainWindow(parent, name), Object(Device_Ide)
{
	Open();

	setMinimumSize(200,200);
	AddNotifiable(this, 0);

	for (int i=0; i<MAX_ATTRIBUTES; i++)
		attribs[i] = 0;
	attribobj = Null;

	for (int i=0; i<MAX_OBJECTS; i++)
		objects[i] = 0;
	objectobj = Null;

	for (int i=0; i<MAX_INPUTS; i++)
		inputs[i] = 0;
	curinput = -1;

	QWidget *cwidget = new QWidget(this);
	setCentralWidget(cwidget);
	mainlayout = new QVBoxLayout(cwidget, 2);

	QToolBar *mainbar = new QToolBar(this);
	mainbar->setLabel("Main Tool Bar");
	QToolButton *runbut = new QToolButton(QIconSet(QPixmap("icons/launch.png")), "Run", QString::null, this, SLOT(fileRun()), mainbar);
	QToolButton *newbut = new QToolButton(QIconSet(QPixmap("icons/filenew.png")), "New", QString::null, this, SLOT(fileNew()), mainbar);
	QToolButton *openbut = new QToolButton(QIconSet(QPixmap("icons/fileopen.png")), "Open", QString::null, this, SLOT(fileOpen()), mainbar);
	QToolButton *savebut = new QToolButton(QIconSet(QPixmap("icons/filesave.png")), "Save", QString::null, this, SLOT(fileSave()), mainbar);
	QToolButton *closebut = new QToolButton(QIconSet(QPixmap("icons/remove.png")), "Close", QString::null, this, SLOT(fileClose()), mainbar);

	splithor = new QSplitter(cwidget);
	objtree = new ObjectTree(splithor);
	splitver = new QSplitter(Vertical, splithor);
	mainlayout->addWidget(splithor);

	itabs = new QTabWidget(splitver);
	screen = new Screen(itabs);
	screen->Open();
	itabs->addTab(screen, QIconSet(QPixmap("icons/dev_screen.png")), "Screen");
	//ObjectWindow *objwindow = new ObjectWindow(Null, itabs);
	//itabs->addTab(objwindow, QIconSet(QPixmap("icons/object.png")), "Object");
	//AttributeWindow *attribwindow = new AttributeWindow(Null, Null, itabs);
	//itabs->addTab(attribwindow, QIconSet(QPixmap("icons/object.png")), "Attribute");

	//Object::GetSystem()->Set(Index::Lookup("screen"), screen->GetOID());
	mtabs = new QTabWidget(splitver);
	msgwin = new MessageWindow(mtabs);
	//inputwin = new InputWindow(mtabs);
	tracewin = new TraceWindow(mtabs);
	monitorwin = new MonitorWindow(mtabs);
	//mtabs->addTab(inputwin, QIconSet(QPixmap("icons/dev_console.png")), "Input");
	mtabs->addTab(0, QIconSet(QPixmap("icons/dev_console.png")),"Output");
	mtabs->addTab(msgwin, QIconSet(QPixmap("icons/message.png")), "Messages");
	mtabs->addTab(tracewin,QIconSet(QPixmap("icons/trace.png")), "Trace");
	mtabs->addTab(monitorwin,QIconSet(QPixmap("icons/monitor.png")),"Monitors");

	MakeMenu();

	QValueList<int> vlist;
	vlist.push_back(100);
	splithor->setSizes(vlist);
	vlist.clear();
	vlist.push_back(400);
	splitver->setSizes(vlist);

	timer = new Clock();
	timer->Open();

	Compiler::ExecuteFile("ide.dos");
}

MainForm::~MainForm()
{
	//timer->Close();
	//delete tracewin;
	//delete inputwin;
	//delete msgwin;
	//delete mtabs;
	//screen->Close();
	//delete objtree;
	//delete splithor;
	//delete splitver;
	//delete file;
	//delete menu;
	//delete mainlayout;
}

void MainForm::MakeMenu()
{
	menu = menuBar();
	statusBar();
	QPopupMenu *file = new QPopupMenu();
	menu->insertItem("&File", file);
	file->insertItem(QIconSet(QPixmap("icons/fileopen.png")), "&Open...", this, SLOT(fileOpen()));
	file->insertSeparator();
	file->insertItem(QIconSet(QPixmap("icons/filesave.png")), "&Save...", this, SLOT(fileSave()));
	file->insertItem(QIconSet(QPixmap("icons/filesaveas.png")), "Save As...");
	file->insertItem(QIconSet(QPixmap("icons/filesaveobj.png")), "S&ave Objects", this, SLOT(fileSaveObjects()));
	file->insertSeparator();
	file->insertItem(QIconSet(QPixmap("icons/run.png")), "&Run...", this, SLOT(fileRun()));
	file->insertSeparator();
	file->insertItem(QIconSet(QPixmap("icons/exit.png")), "&Quit", app, SLOT(quit()));

	edit = new QPopupMenu();
	menu->insertItem("&Edit", edit);
	edit->insertItem(QIconSet(QPixmap("icons/undo.png")), "&Undo", inputwin, SLOT(editUndo()), CTRL+Key_Z, 0);
	edit->insertItem(QIconSet(QPixmap("icons/redo.png")), "Re&do", inputwin, SLOT(editRedo()), 0, 1);
	edit->insertSeparator();
	edit->insertItem(QIconSet(QPixmap("icons/editcut.png")), "Cu&t", inputwin, SLOT(editCut()), CTRL+Key_X, 2);
	edit->insertItem(QIconSet(QPixmap("icons/editcopy.png")), "&Copy", inputwin, SLOT(editCopy()), CTRL+Key_C, 3);
	edit->insertItem(QIconSet(QPixmap("icons/editpaste.png")), "&Paste", inputwin, SLOT(editPaste()), CTRL+Key_V, 4);
	edit->insertSeparator();
	edit->insertItem("Select &All", inputwin, SLOT(editSelectall()), CTRL+Key_A, 5);
	edit->insertItem("Dese&lect", inputwin, SLOT(editClear()), 0, 6);
	edit->insertSeparator();
	edit->insertItem("&Find");
	edit->insertItem("Find &Next");
	edit->insertItem("Find Pre&vious");
	edit->insertItem("&Replace");
	edit->insertSeparator();
	edit->insertItem("&Go to Line");
	connect(edit, SIGNAL(aboutToShow()), this, SLOT(editAboutToShow()));
	
	QPopupMenu *view = new QPopupMenu();
	menu->insertItem("&View", view);
	view->insertItem("Show Object Tree");
	view->insertItem("Detach Screen");
	QPopupMenu *tools = new QPopupMenu();
	menu->insertItem("&Tools", tools);
	QPopupMenu *settings = new QPopupMenu();
	menu->insertItem("&Settings", settings);
	QPopupMenu *help = new QPopupMenu();
	menu->insertItem("&Help", help);
	help->insertItem("About");
	help->insertItem("Contents");
	help->insertItem("Search");
}

void MainForm::editAboutToShow()
{
	if (inputwin->hasSelected() == false)
	{
		edit->setItemEnabled(2, false);
		edit->setItemEnabled(3, false);
		edit->setItemEnabled(6, false);
	} else
	{
		edit->setItemEnabled(2, true);
		edit->setItemEnabled(3, true);
		edit->setItemEnabled(6, true);
	}

	if (inputwin->hasUndo())
		edit->setItemEnabled(0, true);
	else
		edit->setItemEnabled(0, false);

	if (inputwin->hasRedo())
		edit->setItemEnabled(1, true);
	else
		edit->setItemEnabled(1, false);
}

void MainForm::OnGarbage(OID obj)
{
	if (obj == Device_Ide)
	{
		if (attribobj != Null)
		{
			Object *object = Object::Lookup(attribobj);
			if (object != 0)
			{
				object->Open();
				object->RemoveNotifiable(this);
				object->Close();
			}
		}
	}
}

void MainForm::OnChange(OID obj, OID attrib, OID value)
{
	if (obj == Device_Ide)
	{
		if (attrib == Index::Lookup("objects"))
		{
			objectobj = value;
			Object *object = Object::Lookup(value);
			if (object != 0)
			{
				object->Open();
				object->AddNotifiable(this, 0);
				object->Close();
			}
		} 
		else if (attrib == Index::Lookup("attributes"))
		{
			attribobj = value;
			Object *object = Object::Lookup(value);
			if (object != 0)
			{
				object->Open();
				object->AddNotifiable(this, 0);
				object->Close();
			}
		} 
		else if (attrib == Index::Lookup("run"))
		{
			if (GUIFactory::IdentifyType(value,attrib) == OBJTYPE_STRING)
			{
				char buf[100];
				ObjToStr(value, buf, 99);
				Compiler::ExecuteFile(buf);
			}
			//Set(attrib, Null);
		}
		else if (attrib == Index::Lookup("open"))
		{
			if (GUIFactory::IdentifyType(value,attrib) == OBJTYPE_STRING)
			{
				char buf[100];
				ObjToStr(value, buf, 99);
				QString s = QString(buf);
				inputwin->LoadFile(s);
			}
			//Set(attrib, Null);
		}
	} else if (obj == attribobj)
	{
		if (attrib.type == OID_TYPE_INTEGER && attrib.value < MAX_ATTRIBUTES)
		{
			if (value != Null)
			{
				if (attribs[ObjToInt(attrib)] != 0)
					delete attribs[ObjToInt(attrib)];
	
				attribs[ObjToInt(attrib)] = new AttributeWindow(value, attribobj, ObjToInt(attrib), itabs);
				itabs->addTab(attribs[ObjToInt(attrib)], QIconSet(QPixmap("icons/object.png")), "Attribute");
				itabs->showPage(attribs[ObjToInt(attrib)]);
			} else
				attribs[ObjToInt(attrib)] = 0;
		}
	}
	else if (obj == objectobj)
	{
		if (attrib.type == OID_TYPE_INTEGER && attrib.value < MAX_ATTRIBUTES)
		{
			if (value != Null)
			{
				if (objects[ObjToInt(attrib)] != 0)
					delete objects[ObjToInt(attrib)];
	
				objects[ObjToInt(attrib)] = new ObjectWindow(value, objectobj, ObjToInt(attrib), itabs);
				itabs->addTab(objects[ObjToInt(attrib)], QIconSet(QPixmap("icons/object.png")), "Object");
				itabs->showPage(objects[ObjToInt(attrib)]);
			} else
				objects[ObjToInt(attrib)] = 0;
		}
	}
}

void MainForm::fileRun()
{
	QString s = QFileDialog::getOpenFileName(
                    QString::null,
                    "DOSTE Scripts (*.dos)",
                    this,
                    "open file dialog",
                    "Choose a file" );

	Compiler::ExecuteFile(s);
}

void MainForm::fileOpen()
{
	QString s = QFileDialog::getOpenFileName(
                    QString::null,
                    "DOSTE Scripts (*.dos)",
                    this,
                    "open file dialog",
                    "Choose a file" );

	InputWindow *nwin = new InputWindow(itabs);
	itabs->addTab(nwin, QIconSet(QPixmap("icons/edit.png")), "File");
	itabs->showPage(nwin);
	nwin->LoadFile(s);
}

void MainForm::fileSave()
{
	QString s = QFileDialog::getSaveFileName(
                    QString::null,
                    "DOSTE Scripts (*.dos)",
                    this,
                    "save file dialog",
                    "Choose a file" );

	ciwin->SaveFile(s);
}

void MainForm::fileClose()
{
	//itabs->deleteTab(ciwin);
}

void MainForm::fileNew()
{
	ciwin = new InputWindow(itabs);
	itabs->addTab(ciwin, QIconSet(QPixmap("icons/edit.png")), "File");
	itabs->showPage(ciwin);
}

void MainForm::fileSaveObjects()
{

}
