#include <ObjectWindow.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qgroupbox.h>
#include <qlistview.h>
#include <Index.h>
#include <Conversion.h>

using namespace doste;

ObjectWindow::ObjectWindow(OID obj, OID par, int num, QWidget *parent)
{
	this->par = par;
	this->obj = obj;
	this->num = num;

	QVBoxLayout *mainlayout = new QVBoxLayout(this, 5);
	QGroupBox *details = new QGroupBox("Object Details", this);
	mainlayout->addWidget(details);
	attribs = new QListView(this);
	attribs->addColumn("Name");
	attribs->addColumn("Value");
	attribs->addColumn("Def");
	mainlayout->addWidget(attribs);

	Object *object = Object::Lookup(obj);
	if (object != 0)
	{
		object->Open();
		object->AddNotifiable(this, 0);

		int i=0;
		OID cur = object->GetAttribute(i);
		while (cur != Error)
		{
			OnChange(obj, cur, object->Get(cur));
			i++;
			cur = object->GetAttribute(i);
		}

		object->Close();
	}

	QHBoxLayout *buttonlayout = new QHBoxLayout(mainlayout);
	buttonlayout->setAlignment(AlignLeft);
	
	QPushButton *close = new QPushButton(QIconSet(QPixmap("icons/cancel.png")), "Close", this);
	buttonlayout->addWidget(close);
	connect(close, SIGNAL(clicked()), this, SLOT(close()));
}

ObjectWindow::~ObjectWindow()
{

}

void ObjectWindow::OnChange(OID obj, OID key, OID value)
{
	attribs->insertItem(new AttribItem(key, value, attribs));
}

void ObjectWindow::OnGarbage(OID obj)
{
	delete this;
}

void ObjectWindow::close()
{
	Object *object = Object::Lookup(par);
	if (object != 0)
	{
		object->Open();
		object->Set(IntToObj(num), Null);
		object->Close();
	}
}

AttribItem::AttribItem(OID key, OID attrib, QListView *parent)
	: QListViewItem(parent)
{
	Update(key, attrib);
}

AttribItem::AttribItem(OID key, OID attrib, QListViewItem *parent)
	: QListViewItem(parent)
{
	Update(key, attrib);
}

AttribItem::~AttribItem()
{
}

void AttribItem::Update(OID key, OID attrib)
{
	this->key = key;
	this->attrib = attrib;

	char buf[50];
	OIDToString(key, buf, 50);
	setText(0, buf);
	OIDToString(attrib, buf, 50);
	setText(1, buf);
}

OID AttribItem::GetKey()
{
	return key;
}
