#include <Syntax.h>
#include <iostream>

Syntax::Syntax(QTextEdit *textEdit)
	: QSyntaxHighlighter(textEdit)
{
	font_command = QFont("Courier New", 12, QFont::Bold);
	font_object = font_command;
	font_comment = QFont("Courier New", 12, QFont::Normal, true);
	font_normal = QFont("Courier New", 12);
	font_oid = font_object;
	font_string = font_normal;
	font_number = font_normal;
	font_char = font_number;
	color_command = QColor("black");
	color_normal = color_command;
	color_object = QColor(60,120,240);
	color_comment = QColor(130,130,130);
	color_oid = QColor("red");
	color_string = color_oid;
	color_number = QColor("blue");
	color_char = QColor("pink");
}

Syntax::~Syntax()
{

}

int Syntax::highlightParagraph ( const QString & text, int endStateOfLastPara )
{
	unsigned int pos = 0;
	unsigned int beg = 0;
	bool istring;
	int i;

	char buffer[100];

	while (pos < text.length())
	{
		//Remove all white space from the buffer, reguardless of whether its in peek mode.
		while (text[pos] == ' ' || text[pos] == '\n' || text[pos] == '\r' || text[pos] == '\t')
		{
			pos++;
		}
	
		switch (text[pos])
		{
			case COMMENT:	beg = pos;
							pos++;
							while (text[pos] != '\n' && text[pos] != (char)0)			//Remove comment until new line
								pos++;
							setFormat(beg, pos-beg, font_comment, color_comment);
							break;
			case CMD:		beg = pos;
							pos++;
							i = 0;
							while (text[pos] != ' ' && text[pos] != '\t' && text[pos] != (char)0)
							{
								buffer[i] = text[pos];
								pos++;
								i++;
							}
							buffer[i] = 0;
							if (strcmp(buffer, "include") == 0)
								setFormat(beg, pos-beg, font_command, color_command);
							else if (strcmp(buffer, "list") == 0)
								setFormat(beg, pos-beg, font_command, color_command);
							else if (strcmp(buffer, "debug") == 0)
								setFormat(beg, pos-beg, font_command, color_command);
							else if (strcmp(buffer, "query") == 0)
								setFormat(beg, pos-beg, font_command, color_command);
							else if (strcmp(buffer, "using") == 0)
								setFormat(beg, pos-beg, font_command, color_command);
							else if (strcmp(buffer, "def") == 0)
								setFormat(beg, pos-beg, font_command, color_command);
							else if (strcmp(buffer, "print") == 0)
								setFormat(beg, pos-beg, font_command, color_command);
							else if (strcmp(buffer, "hide") == 0)
								setFormat(beg, pos-beg, font_command, color_command);
							else
								setFormat(beg, pos-beg, font_normal, color_normal);
							break;
			case TERM:		break;
			case TLP:
			case RP:
			case LO:
			case RO:
			case RS:
			case LS:		pos++; break;
			case LC:		beg = pos;
							pos++;
							break;
			case RC:		pos++;
							setFormat(beg, pos-beg, font_oid, color_oid);
							break;
			case ASSIGN:	pos++;
							break;
			case SEP:		pos++;
							break;
			case END:		pos++;
							break;
	
			default:		beg = pos;
							i = 0;
							istring = false;
							while (pos < text.length() && (istring || (text[pos] != ' ' && text[pos] != '\t' && text[pos] != (char)0 && text[pos] != '\n' && text[pos] != '(' && text[pos] != ')' && text[pos] != ';' && text[pos] != ',' && text[pos] != '<' && text[pos] != '>')))
							{
								if (text[pos] == '"')
									istring = !istring;
								buffer[i] = text[pos];
								pos++;
								i++;
							}
							buffer[i] = 0;
							if (strcmp(buffer, "system") == 0)
								setFormat(beg, pos-beg, font_object, color_object);
							else if (strcmp(buffer, "current") == 0)
								setFormat(beg, pos-beg, font_object, color_object);
							else if (strcmp(buffer, "this") == 0)
								setFormat(beg, pos-beg, font_object, color_object);
							else if (strcmp(buffer, "true") == 0)
								setFormat(beg, pos-beg, font_object, color_object);
							else if (strcmp(buffer, "false") == 0)
								setFormat(beg, pos-beg, font_object, color_object);
							else if (strcmp(buffer, "null") == 0)
								setFormat(beg, pos-beg, font_object, color_object);
							else if (strcmp(buffer, "*") == 0)
								setFormat(beg, pos-beg, font_object, color_object);
							else if (strcmp(buffer, "$") == 0)
								setFormat(beg, pos-beg, font_object, color_object);
							else
							{
								//Check for numbers, characters and strings
								if (buffer[0] == '"')
									setFormat(beg, pos-beg, font_string, color_string);
								else if (buffer[0] == '0' || buffer[0] == '1' || buffer[0] == '2' || buffer[0] == '3' || buffer[0] == '4' || buffer[0] == '5' || buffer[0] == '6' || buffer[0] == '7' || buffer[0] == '8' || buffer[0] == '9' || buffer[0] == '-')
									setFormat(beg, pos-beg, font_number, color_number);
								else if (buffer[0] == '\'')
									setFormat(beg, pos-beg, font_char, color_char);
								else
									setFormat(beg, pos-beg, font_normal, color_normal);
							}
							break;
		}
	}

	return 0;
}
