#ifndef _COMPILER_
#define _COMPILER_

#include <Object.h>
#include <OID.h>
#include <qstring.h>
#include <CompilerNotifiable.h>

#define CERROR_NULL				0
#define CERROR_UNKNOWN			1
#define CERROR_BADSTATEMENT		2
#define CERROR_INVALASSIGN		3
#define CERROR_NOSTATEMENT		4
#define CERROR_NOOBJECT			5
#define CERROR_SYNTAX			6
#define CERROR_MISSINGRP		7
#define CERROR_MISSINGRC		8
#define CERROR_COMPORDER		9
#define CERROR_LHSCOMP			10
#define CERROR_MISSINGVALUE		11
#define CERROR_NULLOBJECT		12
#define CERROR_FILENAME			13

#define CMD_UNKNOWN		0
#define CMD_INCLUDE		1
#define CMD_LIST		2
#define CMD_DEF			3
#define CMD_USING		4
#define CMD_QUERY		5
#define CMD_DEBUG		6
#define CMD_PRINT		7
#define CMD_HIDE		8

using namespace doste;

struct Session
{
	//bool file;
	const char *buffer;
	const char *curbuf;
	bool lhs;
	OID thi;
	//bool comp;
	//int errorcode;
	OID tokval;
	OID key;
	int inset;
	bool critical;
	bool errstatement;
	const char *filename;
	int line;
	bool query;
	bool getdef;
	vector<OID> *def;
	bool debug;
};

enum Token {TLP='(', RP=')', LS='{', RS='}', LC='[', RC=']', COMMENT='#', LO='<', RO='>', ASSIGN='=',
				STRING='"', CHARSTART='\'', OIDSEP=':', END=';', TERM=0, OBJECT='O', SEP=',', CMD='%'};

namespace doste
{
	class Compiler
	{
		public:
			static void Init();
			static void Final();

			static OID Execute(const char *buffer, const char *filename="console");
			static OID ExecuteFile(const char *filename);
			static vector<OID> *CompileDefinition(const char *buffer, OID parent);
			static OID EvalDef(vector<OID> *def);

			static void SetCurrent(OID cur);
			static OID GetCurrent();

			static void SetNotifiable(CompilerNotifiable *notif);

		private:
			static Token GetToken(Session *s, bool peek);
			static OID TranslateObject(Session *s, bool peek);
			static OID TranslateOID(char *word);

			static OID ParseStatement(Session *s);
			static vector<OID> *ParseObject(Session *s, OID self);
			static OID ParseSet(Session *s);
			static void ParseFilename(Session *s, char *buffer);
			static int ParseCommand(Session *s);

			static void PrintError(Session *s, const QString &error);
			static void PrintWarning(Session *s, const QString &warning);

			static OID m_current;
			static CompilerNotifiable *notif;
	};
};

#endif
