#ifndef _OBJECT_
#define _OBJECT_

#include <OID.h>
#include <list>
#include <ObjectNotifiable.h>
#include <Observable.h>

#include <pthread.h>

using namespace doste;

struct RAttribute
{
	OID key;
	Observable *value;
};

struct VAttribute
{
	OID key;
	Observable *vkey;
};

namespace doste
{
	class Object
	{
		friend class Obeservable;

		public:
			Object();
			Object(OID o);
			virtual ~Object();

			inline const OID &GetOID() { return oid; };
			void Open();
			void Close(bool garbageable = true);

			virtual OID Get(OID key);
			virtual void Set(OID key, OID value);
			void Set(vector<OID> *key, OID value);
			virtual void Set(OID key, vector<OID> *def);
			void Set(vector<OID> *key, vector<OID> *def);
			virtual vector<OID> *GetDefinition(OID key);
			virtual list<SDependant> *GetDependants(OID key);

			void QSet(OID key, OID value);

			unsigned char GetFlags(OID key);
			void SetFlags(OID key, unsigned char flags);

			virtual OID GetAttribute(int index);
			//virtual void Rename(OID key, OID newkey);
			//virtual void Remove(OID key);
			int GetNumReferences();

			//These are special, used but obeservables only.
			virtual void Notify(OID key, bool forced);
			virtual void AddDependant(OID key, OID obj, OID attr);
			virtual void RemoveDependant(OID key, OID obj, OID attr);

			//static void SetGlobalNotifiable(ObjectNotifiable *notif, unsigned int flags);
			void AddNotifiable(ObjectNotifiable *notif, unsigned int flags);
			void RemoveNotifiable(ObjectNotifiable *notif);

			static Object *Lookup(OID o);
			//static Object *NewObject();
			//static Object *NewObject(OID o);
			//static void RecycleObject(Object *obj);
			static void Init();
			static void Final();
			static Object *GetSystem();
			static unsigned int GetNumObjects();

			static void SetCurrent(OID current);
			static OID GetCurrent();

			struct SQueue
			{
				OID key;
				OID value;
				Object *obj;
			};

		private:
			short exrefs;
			OID oid;
			OID parent;
			list<RAttribute*> **attribs;
			list<VAttribute*> *vattribs;
			list<ObjectNotifiable*> *notifiables;

			Observable *LookupObservable(OID key);
			void AddRAttribute(RAttribute *attrib);
			void AddVAttribute(VAttribute *attrib);
			void RemoveRAttribute(OID key);

			//void RealSet(OID key, vector<OID> *def);
			static void *QueueProcessor(void *p);

			inline static int HashOID(OID o, int size);

			static const int PAIR_HASH_SIZE = 20;
			static const int GLOBAL_HASH_SIZE = 1000;
			static const int MAX_QUEUE_SIZE = 500;

			static void AddObject(Object *obj);
			static void RemoveObject(Object *obj);
			static OID AllocateOID();
			static OID GenVKey();
			static SQueue *AllocateQueueEntry();
			static void FreeQueueEntry(SQueue *q);

			static OID lastvkey;
			static Object *system;
			static bool active;
			static OID lastoid;
			static OID currentobj;
			static list<Object*> **objects;
			static ObjectNotifiable *gnotif;
			static list<Object*> *recycled;
			static unsigned int m_numactiveobjects;
			static list<SQueue*> setqueue;
			static pthread_t qthread;
			static pthread_mutex_t qmutex;
			static pthread_cond_t qcond;
			static bool iswaiting;
			static list<SQueue*> freequeue;
	};
};

#endif 
