#ifndef _OBJECTTREE_
#define _OBJECTTREE_

#include <qvbox.h>
#include <qlistview.h>
#include <ObjectNotifiable.h>
#include <qstring.h>
#include <qpopupmenu.h>
#include <AttribNameDialog.h>
#include <qdragobject.h>

class ObjectItem : public QListViewItem, public ObjectNotifiable
{
	public:
		ObjectItem(QListView *parent, OID o, OID attrib, OID value, QString name, bool def);
		ObjectItem(ObjectItem *parent, OID o, OID attrib, OID value, QString name, bool def);
		~ObjectItem();

		void Update(OID parent, OID attrib, OID value, QString name, bool def);

		OID GetObject() { return pobject; };
		OID GetAttrib() { return attrib; };
		OID GetValue() { return object; };

		void OnChange(OID o, OID attrib, OID value);
		void Expand();
		void Collapse();

	private:
		OID object;
		OID attrib;
		OID pobject;
		QPixmap *pm_open;
		QPixmap *pm_closed;
};

class ObjectListView : public QListView
{
	public:
		ObjectListView(QWidget *parent);
		~ObjectListView();

		QString GeneratePath(QListViewItem* item);

	private:
		void startDrag();
};

class ObjectTree : public QVBox
{
	Q_OBJECT

	public:
		ObjectTree(QWidget *parent);
		~ObjectTree();

	private:
		ObjectListView *list;
		QPopupMenu *menu;
		ObjectItem *popitem;
		AttribNameDialog *aname;
		bool copying;
		bool cutting;
		bool referencing;
		QString copypath;

	private slots:
		void onItem(QListViewItem *i);
		void itemClicked(QListViewItem *i);
		void expanded(QListViewItem *item);
		void collapsed(QListViewItem *item);
		void itemRenamed(QListViewItem *item, int col, const QString &text);
		void rightButtonClicked(QListViewItem *item, const QPoint &pos, int col);

		void addAttribute();
		void deleteObject();
		void saveObject();
		void monitorObject();
		void objcopy();
		void objcut();
		void objref();
		void traceObject();
		void editObject();
		void viewObject();
};

#endif
