#ifndef _NUMBERS_
#define _NUMBERS_

#include <Object.h>
#include <Index.h>
#include <Conversion.h>
#include <iostream>

namespace doste
{
	class AddObject : public Object
	{
		public:
			AddObject(OID parent) : Object()
			{
				Open();
				QSet(Parent, parent);
				Close(false);
			};

			~AddObject() {};

			OID Get(OID attrib)
			{
				if (attrib.type != OID_TYPE_INTEGER && attrib.type != OID_TYPE_FLOAT)
					return Object::Get(attrib);

				OID parent = Get(Parent);

				if (parent.type == OID_TYPE_INTEGER)
				{
					return IntToObj(ObjToInt(parent) + ObjToInt(attrib));
				} else if (parent.type == OID_TYPE_FLOAT)
				{
					return FloatToObj(ObjToFloat(parent) + ObjToFloat(attrib));
				} else
				{
					return Object::Get(attrib);
				}
			};

			void AddDependant(OID o1, OID o2, OID o3) {};
	};

	class SubObject : public Object
	{
		public:
			SubObject(OID parent) : Object()
			{
				Open();
				QSet(Parent, parent);
				Close(false);
			};
			~SubObject() {};

			OID Get(OID attrib)
			{
				if (attrib.type != OID_TYPE_INTEGER && attrib.type != OID_TYPE_FLOAT)
					return Object::Get(attrib);

				OID parent = Get(Parent);

				if (parent.type == OID_TYPE_INTEGER)
				{
					return IntToObj(ObjToInt(parent) - ObjToInt(attrib));
				} else if (parent.type == OID_TYPE_FLOAT)
				{
					return FloatToObj(ObjToFloat(parent) - ObjToFloat(attrib));
				} else
				{
					return Object::Get(attrib);
				}
			};

			void AddDependant(OID o1, OID o2, OID o3) {};
	};

	class MulObject : public Object
	{
		public:
			MulObject(OID parent) : Object()
			{
				Open();
				QSet(Parent, parent);
				Close(false);
			};
			~MulObject() {};

			OID Get(OID attrib)
			{
				if (attrib.type != OID_TYPE_INTEGER && attrib.type != OID_TYPE_FLOAT)
					return Object::Get(attrib);

				OID parent = Get(Parent);

				if (parent.type == OID_TYPE_INTEGER)
				{
					return IntToObj(ObjToInt(parent) * ObjToInt(attrib));
				} else if (parent.type == OID_TYPE_FLOAT)
				{
					return FloatToObj(ObjToFloat(parent) * ObjToFloat(attrib));
				} else
				{
					return Object::Get(attrib);
				}
			};

			void AddDependant(OID o1, OID o2, OID o3) {};
	};

	class DivObject : public Object
	{
		public:
			DivObject(OID parent) : Object()
			{
				Open();
				QSet(Parent, parent);
				Close(false);
			};
			~DivObject() {};

			OID Get(OID attrib)
			{
				if (attrib.type != OID_TYPE_INTEGER && attrib.type != OID_TYPE_FLOAT)
					return Object::Get(attrib);

				OID parent = Get(Parent);

				if (parent.type == OID_TYPE_INTEGER)
				{
					if (ObjToInt(attrib) == 0)
						return Null;
					return IntToObj(ObjToInt(parent) / ObjToInt(attrib));
				} else if (parent.type == OID_TYPE_FLOAT)
				{
					return FloatToObj(ObjToFloat(parent) / ObjToFloat(attrib));
				} else
				{
					return Object::Get(attrib);
				}
			};

			void AddDependant(OID o1, OID o2, OID o3) {};
	};

	class LessObject : public Object
	{
		public:
			LessObject(OID parent) : Object()
			{
				Open();
				QSet(Parent, parent);
				Close(false);
			};
			~LessObject() {};

			OID Get(OID attrib)
			{
				if (attrib.type != OID_TYPE_INTEGER && attrib.type != OID_TYPE_FLOAT)
					return Object::Get(attrib);

				OID parent = Get(Parent);

				if (parent.type == OID_TYPE_INTEGER)
				{
					return BoolToObj(ObjToInt(parent) < ObjToInt(attrib));
				} else if (parent.type == OID_TYPE_FLOAT)
				{
					return BoolToObj(ObjToFloat(parent) < ObjToFloat(attrib));
				} else
				{
					return Object::Get(attrib);
				}
			};

			void AddDependant(OID o1, OID o2, OID o3) {};
	};
};

#endif
