#include <objects/DOpenGL.h>
#include <GUIFactory.h>
#include <Index.h>
#include <Conversion.h>
#include <iostream>
#include <stdio.h>
#include <stdlib.h>

DOpenGL::DOpenGL(OID par, OID key, OID obj, QWidget *parent)
	: DWidget(par,key,obj), QGLWidget(parent)
{
	LoadObject();
	show();
}

DOpenGL::~DOpenGL()
{
}

void DOpenGL::LoadTexture(OID obj)
{
	char filename[200];
	ObjToStr(obj,filename,200);

	unsigned char TGAheader[12] = {0,0,2,0,0,0,0,0,0,0,0,0};
	unsigned char TGAcompare[12];
	unsigned char header[6];
	unsigned int bytesperpixel;
	unsigned int imagesize;
	unsigned int temp;
	unsigned int texid;

	FILE *fhandle = fopen(filename, "rb");
	if (fhandle <= 0)
	{
		printf("File not found: %s\n", filename);
		return;
	}

	std::cout << "File Opened\n";

	fread((char*)TGAcompare, 1, sizeof(TGAcompare), fhandle);

	fread((char*)header, 1, sizeof(header), fhandle);

	int width = header[1] * 256 + header[0];
	int height = header[3] * 256 + header[2];

	if (header[4] != 24 && header[4] != 32)
	{
		printf("Invalid TGA File: %s\n", filename);
		fclose(fhandle);
		return;
	}

	std::cout << "Header Read\n";

	int bpp = header[4];
	bytesperpixel = bpp / 8;
	imagesize = width * height * bytesperpixel;

	unsigned char *data = new unsigned char[imagesize];
	fread((char*)data, 1, imagesize, fhandle);

	for (int i=0; i<imagesize; i+= bytesperpixel)
	{
		temp=data[i];
		data[i] = data[i+2];
		data[i+2] = temp;
	}

	std::cout << "Data read and processed\n";

	fclose(fhandle);

	glGenTextures(1, &texid);
	glBindTexture(GL_TEXTURE_2D, texid);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

	int type;
	if (bpp == 24)
		type = GL_RGB;
	if (bpp == 32)
		type = GL_RGBA;

	glTexImage2D(GL_TEXTURE_2D, 0, type, width, height, 0, type, GL_UNSIGNED_BYTE, data);

	std::cout << "Texture generated\n";

	delete [] data;

	Object *tex = Object::Lookup(obj);
	if (tex != 0)
	{
		tex->Open();
		tex->Set(Index::Lookup("glid"), IntToObj(texid));
		tex->Close();
	}

	std::cout << "Done\n";
}

void DOpenGL::OnChange(OID obj, OID attrib, OID value)
{
	if (attrib == Parent)
		return;
	if (attrib == Index::Lookup("widget"))
		return;

	if (obj == texobj)
	{
		//Assume everything is a texture object.
		if (GUIFactory::IdentifyType(value, attrib) == OBJTYPE_STRING)
		{
			std::cout << "Load Texture\n";
			LoadTexture(value);
		}
	}
	else if (obj == object)
	{
		if (attrib == Index::Lookup("textures"))
		{
			texobj = value;
			Object *tex = Object::Lookup(texobj);
			if (tex != 0)
			{
				tex->Open();
				tex->AddNotifiable(this, 0);
				int i=0;
				OID key = tex->GetAttribute(i);
				while (key != Error)
				{
					OnChange(texobj, key, tex->Get(key));
					i++;
					key = tex->GetAttribute(i);
				}
				tex->Close();
			}
		}
		else if (attrib == Index::Lookup("width"))
		{
			if (value.type == OID_TYPE_INTEGER)
				resize(value.value, height());
		}
		else if (attrib == Index::Lookup("height"))
		{
			if (value.type == OID_TYPE_INTEGER)
				resize(width(), value.value);
		}
		else if (attrib == Index::Lookup("draw"))
		{
			if (value == True)
				updateGL();
		}
		else if (attrib == Index::Lookup("x"))
		{
			if (value.type == OID_TYPE_INTEGER)
				move(value.value, y());
		}
		else if (attrib == Index::Lookup("y"))
		{
			if (value.type == OID_TYPE_INTEGER)
				move(x(), value.value);
		}
		else
		{
			Object *obje = Object::Lookup(value);
			if (obje != 0)
			{
				obje->Open();
				OID wtype = obje->Get(Index::Lookup("entity"));
				if (wtype == Error || wtype == Null)
				{
					obje->Close();
					return;
				}
	
				//Deal with shapes here.
				Entity *nent = Entity::CreateEntity(wtype, obj, attrib, value);
				if (nent != 0)
					entities.push_front(nent);
				
				obje->Close();
			}
		}
	}
}

void DOpenGL::initializeGL()
{
	glShadeModel(GL_SMOOTH);                // Enable smooth shading
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);   // Black background
	glClearDepth(1.0f);                     // Depth buffer setup
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_DEPTH_TEST);                // Enables depth testing
	glEnable(GL_BLEND);
	glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
	glDepthFunc(GL_LEQUAL);                 // The type of depth testing to do
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);      // Really nice perspective calculations
        //glEnable(GL_LIGHTING);
        //glEnable(GL_LIGHT0);
}

void DOpenGL::resizeGL(int width, int height)
{
	glViewport(0,0,width, height);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(40.0, (float)width/(float)height, 1.0, 80.0);
	glMatrixMode(GL_MODELVIEW);
}

void DOpenGL::paintGL()
{
	glClear(GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT);
	glLoadIdentity();
	glTranslatef(0.0,0.0,-3.0);
	
	for (list<Entity*>::iterator i=entities.begin(); i!=entities.end(); i++)
	{
		if ((*i)->IsGarbage())
		{
			delete *i;
			entities.erase(i);
		}
		else
			(*i)->Draw();
	}
}
