#include <Entities.h>
#include <qgl.h>
#include <Index.h>
#include <Conversion.h>
#include <iostream>

Entity *Entity::CreateEntity(OID type, OID parent, OID key, OID object)
{
	if (type == Index::Lookup("polygon"))
		return new Polygon(parent, key, object);
	if (type == Index::Lookup("custom"))
		return new CustomEntity(parent, key, object);

	return 0;
}

Polygon::Polygon(OID par, OID key, OID object)
	: Entity(par, key, object)
{
	rotobj = Null;
	posobj = Null;
	texobj = Null;

	Object *poly = Object::Lookup(object);
	if (poly != 0)
	{
		poly->Open();
		poly->AddNotifiable(this, 0);
		int i=0;
		OID key = poly->GetAttribute(i);
		while (key != Error)
		{
			OnChange(object, key, poly->Get(key));
			i++;
			key = poly->GetAttribute(i);
		}
		poly->Close();
	}
	garbage = false;
}

Polygon::~Polygon()
{
	for (vector<PolyVertex*>::iterator i=vertices.begin(); i!=vertices.end(); i++)
	{
		delete (*i);
	}
}

void Polygon::Draw()
{
	glPushMatrix();

	if (posobj != Null)
		glTranslatef(position.x, position.y, position.z);
	if (rotobj != Null)
	{
		glRotatef(rotation.x, 1.0, 0.0, 0.0);
		glRotatef(rotation.y, 0.0, 1.0, 0.0);
		glRotatef(rotation.z, 0.0, 0.0, 1.0);
	}

	if (texobj != Null)
	{
		glBindTexture(GL_TEXTURE_2D, texid);
		std::cout << "Texid: " << texid << std::endl;
	}

	glBegin(GL_POLYGON);
		for (vector<PolyVertex*>::iterator i=vertices.begin(); i!=vertices.end(); i++)
		{
			if ((*i) != 0)
			{
				if (texobj != Null)
					glTexCoord2f((*i)->u, (*i)->v);
				glColor3f((*i)->col.x, (*i)->col.y, (*i)->col.z);
				glVertex3f((*i)->pos.x, (*i)->pos.y, (*i)->pos.z);
			}
		}
	glEnd();

	glPopMatrix();
}

void Polygon::OnGarbage(OID obj)
{
	if (obj == this->obj)
	{
		garbage = true;
	}
}

void Polygon::OnChange(OID obj, OID key, OID value)
{
	if (key == Parent)
		return;
	if (key == Index::Lookup("entity"))
		return;

	if (obj == posobj)
	{
		if (key == Index::Lookup("x"))
			position.x = ObjToFloat(value);
		else if (key == Index::Lookup("y"))
			position.y = ObjToFloat(value);
		else if (key == Index::Lookup("z"))
			position.z = ObjToFloat(value);
	}

	if (obj == rotobj)
	{
		if (key == Index::Lookup("x"))
			rotation.x = ObjToFloat(value);
		else if (key == Index::Lookup("y"))
			rotation.y = ObjToFloat(value);
		else if (key == Index::Lookup("z"))
			rotation.z = ObjToFloat(value);
	}

	if (obj == texobj)
	{
		if (key == Index::Lookup("glid"))
		{
			texid = ObjToInt(value);
			std::cout << "Setting Texture\n";
		}
	}

	if (obj == this->obj)
	{
		if (key == Index::Lookup("texture"))
		{
			Object *tex = Object::Lookup(value);
			if (tex != 0)
			{
				tex->Open();
				texid = ObjToInt(tex->Get(Index::Lookup("glid")));
				std::cout << "Setting Texture\n";
				tex->Close();
				texobj = value;
			}

			//If display list then may need to rebuild it!
		}
		else if (key == Index::Lookup("position"))
		{
			Object *pos = Object::Lookup(value);
			posobj = value;
			if (pos != 0)
			{
				pos->Open();
				pos->AddNotifiable(this, 0);
				position.x = ObjToFloat(pos->Get(Index::Lookup("x")));
				position.y = ObjToFloat(pos->Get(Index::Lookup("y")));
				position.z = ObjToFloat(pos->Get(Index::Lookup("z")));
				pos->Close();
			}
		}
		else if (key == Index::Lookup("rotation"))
		{
			Object *rot = Object::Lookup(value);
			rotobj = value;
			if (rot != 0)
			{
				rot->Open();
				rot->AddNotifiable(this, 0);
				rotation.x = ObjToFloat(rot->Get(Index::Lookup("x")));
				rotation.y = ObjToFloat(rot->Get(Index::Lookup("y")));
				rotation.z = ObjToFloat(rot->Get(Index::Lookup("z")));
				rot->Close();
			}
		}
		else
		{
			if (key.type == OID_TYPE_INTEGER && key.value < 10)
			{
				if (vertices.size() > key.value && vertices[key.value] != 0)
				{
					delete vertices[key.value];
					vertices[key.value] = 0;
				}
	
				if (vertices.size() <= key.value)
					vertices.resize(key.value+1, 0);
				//Need to check that it is a valid vertex.
				PolyVertex *npoly = new PolyVertex(value);
				vertices[key.value] = npoly;
			}
		}
	}
}

PolyVertex::PolyVertex(OID obj)
{
	this->obj = obj;

	col.x = 1.0;
	col.y = 1.0;
	col.z = 1.0;

	Object *vert = Object::Lookup(obj);
	if (vert != 0)
	{
		vert->Open();
		vert->AddNotifiable(this, 0);
		int i=0;
		OID key = vert->GetAttribute(i);
		while (key != Error)
		{
			OnChange(obj, key, vert->Get(key));
			i++;
			key = vert->GetAttribute(i);
		}
		vert->Close();
	}
}

PolyVertex::~PolyVertex()
{
}

void PolyVertex::OnChange(OID obj, OID key, OID value)
{
	if (key == Parent)
		return;

	if (key == Index::Lookup("x"))
		pos.x = ObjToFloat(value);
	else if (key == Index::Lookup("y"))
		pos.y = ObjToFloat(value);
	else if (key == Index::Lookup("z"))
		pos.z = ObjToFloat(value);

	else if (key == Index::Lookup("r"))
		col.x = ObjToFloat(value);
	else if (key == Index::Lookup("g"))
		col.y = ObjToFloat(value);
	else if (key == Index::Lookup("b"))
		col.z = ObjToFloat(value);

	else if (key == Index::Lookup("u"))
		u = ObjToFloat(value);
	else if (key == Index::Lookup("v"))
		v = ObjToFloat(value);
}

CustomEntity::CustomEntity(OID par, OID key, OID obj)
	: Entity(par,key,obj)
{
	posobj = Null;
	rotobj = Null;

	displist = false;
	haslist = false;

	Object *poly = Object::Lookup(obj);
	if (poly != 0)
	{
		poly->Open();
		poly->AddNotifiable(this, 0);
		int i=0;
		OID key = poly->GetAttribute(i);
		while (key != Error)
		{
			OnChange(obj, key, poly->Get(key));
			i++;
			key = poly->GetAttribute(i);
		}
		poly->Close();
	}
}

CustomEntity::~CustomEntity()
{
	for (list<Entity*>::iterator i=entities.begin(); i!=entities.end(); i++)
	{
		delete (*i);
	}
}

void CustomEntity::OnChange(OID obj, OID key, OID value)
{
	if (key == Parent)
		return;
	if (key == Index::Lookup("entity"))
		return;

	if (obj == posobj)
	{
		if (key == Index::Lookup("x"))
			position.x = ObjToFloat(value);
		else if (key == Index::Lookup("y"))
			position.y = ObjToFloat(value);
		else if (key == Index::Lookup("z"))
			position.z = ObjToFloat(value);
	}

	if (obj == rotobj)
	{
		if (key == Index::Lookup("x"))
			rotation.x = ObjToFloat(value);
		else if (key == Index::Lookup("y"))
			rotation.y = ObjToFloat(value);
		else if (key == Index::Lookup("z"))
			rotation.z = ObjToFloat(value);
	}

	if (obj == this->obj)
	{
		if (key == Index::Lookup("static"))
		{
			displist = ObjToBool(value);
			if (haslist)
			{
				glDeleteLists(gllist, 1);
				haslist = false;
			}
		}
		else if (key == Index::Lookup("position"))
		{
			Object *pos = Object::Lookup(value);
			posobj = value;
			if (pos != 0)
			{
				pos->Open();
				pos->AddNotifiable(this, 0);
				position.x = ObjToFloat(pos->Get(Index::Lookup("x")));
				position.y = ObjToFloat(pos->Get(Index::Lookup("y")));
				position.z = ObjToFloat(pos->Get(Index::Lookup("z")));
				pos->Close();
			}
		}
		else if (key == Index::Lookup("rotation"))
		{
			Object *rot = Object::Lookup(value);
			rotobj = value;
			if (rot != 0)
			{
				rot->Open();
				rot->AddNotifiable(this, 0);
				rotation.x = ObjToFloat(rot->Get(Index::Lookup("x")));
				rotation.y = ObjToFloat(rot->Get(Index::Lookup("y")));
				rotation.z = ObjToFloat(rot->Get(Index::Lookup("z")));
				rot->Close();
			}
		} else
		{

			for (list<Entity*>::iterator i=entities.begin(); i!=entities.end(); i++)
			{
				if ((*i)->key == key)
				{
					delete (*i);
					entities.erase(i);
					break;
				}
			}
		
			Object *val = Object::Lookup(value);
			if (val != 0)
			{
				val->Open();
				OID etype = val->Get(Index::Lookup("entity"));
				if (etype != Null)
				{
					Entity *nent = CreateEntity(etype, obj, key, value);
					if (nent != 0)
					{
						entities.push_front(nent);
					}
				}
				val->Close();
			}
		}
	}
}

void CustomEntity::Draw()
{
	glPushMatrix();

	if (posobj != Null)
		glTranslatef(position.x, position.y, position.z);
	if (rotobj != Null)
	{
		glRotatef(rotation.x, 1.0, 0.0, 0.0);
		glRotatef(rotation.y, 0.0, 1.0, 0.0);
		glRotatef(rotation.z, 0.0, 0.0, 1.0);
	}

	if (displist && !haslist)
	{
		gllist = glGenLists(1);
		glNewList(gllist, GL_COMPILE);

		for (list<Entity*>::iterator i=entities.begin(); i!=entities.end(); i++)
		{
			(*i)->Draw();
		}

		glEndList();
		haslist = true;

	} else if (displist)
	{
		glCallList(gllist);
	} else
	{
		for (list<Entity*>::iterator i=entities.begin(); i!=entities.end(); i++)
		{
			(*i)->Draw();
		}
	}

	glPopMatrix();
}
