#include <objects/Screen.h>
#include <GUIFactory.h>
#include <Conversion.h>
#include <Index.h>
#include <iostream>

Screen::Screen(QWidget *parent)
	: QWidget(parent), Object(Device_Screen)
{
	setCaption("DOSTE Screen");
	setGeometry(0,0,400,400);
	setAcceptDrops(true);
	show();

	AddNotifiable(this, 0);

	children = new list<DWidget*>();
}

Screen::~Screen()
{
	//for (list<DWidget*>::iterator i=children->begin(); i!=children->end(); i++)
	//	delete *i;
	delete children;
}

void Screen::resizeEvent(QResizeEvent *event)
{
	Set(Index::Lookup("width"), IntToObj(event->size().width()));
	Set(Index::Lookup("height"), IntToObj(event->size().height()));
}

void Screen::dragEnterEvent(QDragEnterEvent *e)
{
	e->accept(QTextDrag::canDecode(e));
}

void Screen::dropEvent(QDropEvent *e)
{
	QString path;
	if (QTextDrag::decode(e, path))
	{
		std::cout << "PUT OBJECT HERE: " << path << std::endl;
	}
}

void Screen::OnChange(OID obj, OID attrib, OID value)
{
	if (attrib == Parent)
		return;

	if (attrib == Index::Lookup("width"))
	{
		if (value.type == OID_TYPE_INTEGER)
			resize(value.value, height());
	}
	else if (attrib == Index::Lookup("height"))
	{
		if (value.type == OID_TYPE_INTEGER)
			resize(width(), value.value);
	}
	else if (attrib == Index::Lookup("caption"))
	{
		char buf[1000];
		setCaption(ObjToStr(value, buf, 1000));
	}
	else if (attrib == Index::Lookup("bgcolour"))
	{
		char buf[1000];
		setPaletteBackgroundColor(QColor(ObjToStr(value, buf, 1000)));
	}
	else
	{
		//It might be a widget window... so make a new widget.
		//Check type.
		//Make widget and add to child list.
		//Set notifier.

		//Need to allow for removal of widgets by setting them to null or something.

		for (list<DWidget*>::iterator i=children->begin(); i!=children->end(); i++)
		{
			if ((*i)->GetKey() == attrib)
			{
				delete (*i);
				children->erase(i);
				break;
			}
		}

		Object *obje = Object::Lookup(value);
		if (obje != 0)
		{
			obje->Open();
			OID wtype = obje->Get(Index::Lookup("widget"));
			if (wtype == Error || wtype == Null)
			{
				obje->Close();
				return;
			}

			DWidget *widget = GUIFactory::CreateWidget(wtype, obj, attrib, value, this);

			if (widget != 0)
			{
				children->push_front(widget);
			}
			obje->Close();
		}
	}
}
